@@ -1,4 +1,5 @@
-from typing import Any, Sequence
+"""Schematic Exceptions"""
+from typing import Any, Optional, Sequence
 
 
 class MissingConfigValueError(Exception):
@@ -12,7 +13,9 @@ class MissingConfigValueError(Exception):
         message.
     """
 
-    def __init__(self, config_keys: Sequence[Any], message: str = None) -> str:
+    def __init__(
+        self, config_keys: Sequence[Any], message: Optional[str] = None
+    ) -> None:
         config_keys_str = " > ".join(config_keys)
         self.message = (
             "The configuration value corresponding to the argument "
@@ -25,7 +28,33 @@ class MissingConfigValueError(Exception):
 
         super().__init__(self.message)
 
-    def __str__(self):
+    def __str__(self) -> str:
+        return f"{self.message}"
+
+
+class WrongEntityTypeError(Exception):
+    """Exception raised when the entity type is not desired
+
+    Args:
+        entity id: For synapse, thi
+        message: custom/pre-defined error message to be returned.
+
+    Returns:
+        message.
+    """
+
+    def __init__(self, syn_id: str, message: Optional[str] = None) -> None:
+        self.message = (
+            f"'{syn_id}'' is not a desired entity type"
+            "Please ensure that you put in the right syn_id"
+        )
+
+        if message:
+            self.message = message
+
+        super().__init__(self.message)
+
+    def __str__(self) -> str:
         return f"{self.message}"
 
 
@@ -42,8 +71,8 @@ class MissingConfigAndArgumentValueError(Exception):
     """
 
     def __init__(
-        self, arg_name: str, config_keys: Sequence[Any], message: str = None
-    ) -> str:
+        self, arg_name: str, config_keys: Sequence[Any], message: Optional[str] = None
+    ) -> None:
         config_keys_str = " > ".join(config_keys)
         self.message = (
             f"The value corresponding to the CLI argument '--{arg_name}'"
@@ -57,7 +86,7 @@ class MissingConfigAndArgumentValueError(Exception):
 
         super().__init__(self.message)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return f"{self.message}"
 
 
@@ -72,7 +101,7 @@ class AccessCredentialsError(Exception):
         message.
     """
 
-    def __init__(self, project: str, message: str = None) -> str:
+    def __init__(self, project: str, message: Optional[str] = None) -> None:
         self.message = (
             f"Your access to '{project}'' could not be resolved. "
             "Please check your credentials and try again."
@@ -83,5 +112,5 @@ class AccessCredentialsError(Exception):
 
         super().__init__(self.message)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return f"{self.message}"
