@@ -1,36 +1,29 @@
-import json
 import logging
-import string
-
-import numpy as np
-import pandas as pd
-import re
-import networkx as nx
-from jsonschema import Draft7Validator, exceptions, validate, ValidationError, FormatError
+import os
 from os.path import exists
 
 # allows specifying explicit variable types
-from typing import Any, Dict, Optional, Text, List
+from typing import Any, Dict, List, Optional, Text
 
-# handle schema logic; to be refactored as SchemaExplorer matures into a package
-# as collaboration with Biothings progresses
+import networkx as nx
+from jsonschema import ValidationError
+from opentelemetry import trace
 
-from schematic.schemas.explorer import SchemaExplorer
 from schematic.manifest.generator import ManifestGenerator
-from schematic.schemas.generator import SchemaGenerator
+from schematic.models.validate_manifest import validate_all
+from schematic.schemas.data_model_graph import DataModelGraph, DataModelGraphExplorer
+from schematic.schemas.data_model_json_schema import DataModelJSONSchema
+from schematic.schemas.data_model_parser import DataModelParser
 
-#TODO: This module should only be aware of the store interface
+# TODO: This module should only be aware of the store interface
 # we shouldn't need to expose Synapse functionality explicitly
 from schematic.store.synapse import SynapseStorage
-
 from schematic.utils.df_utils import load_df
 
-from schematic.models.validate_attribute import ValidateAttribute
-from schematic.models.validate_manifest import validate_all
-
-
 logger = logging.getLogger(__name__)
 
+tracer = trace.get_tracer("Schematic")
+
 
 class MetadataModel(object):
     """Metadata model wrapper around schema.org specification graph.
@@ -43,8 +36,12 @@ class MetadataModel(object):
         - generate validation schema view of the metadata model
     """
 
-    def __init__(self, inputMModelLocation: str, inputMModelLocationType: str,) -> None:
-
+    def __init__(
+        self,
+        inputMModelLocation: str,
+        inputMModelLocationType: str,
+        data_model_labels: str,
+    ) -> None:
         """Instantiates a MetadataModel object.
 
         Args:
@@ -53,17 +50,26 @@ class MetadataModel(object):
         """
         # extract extension of 'inputMModelLocation'
         # ensure that it is necessarily pointing to a '.jsonld' file
-        if inputMModelLocation.rpartition(".")[-1] == "jsonld":
-            logger.debug(
-                f"Initializing SchemaGenerator object from {inputMModelLocation} schema."
-            )
-            self.inputMModelLocation = inputMModelLocation
 
-            self.sg = SchemaGenerator(inputMModelLocation)
-        else:
-            raise TypeError(
-                f"Please make sure {inputMModelLocation} is a .jsonld file."
-            )
+        logger.debug(
+            f"Initializing DataModelGraphExplorer object from {inputMModelLocation} schema."
+        )
+
+        # self.inputMModelLocation remains for backwards compatibility
+        self.inputMModelLocation = inputMModelLocation
+        self.path_to_json_ld = inputMModelLocation
+
+        data_model_parser = DataModelParser(path_to_data_model=self.inputMModelLocation)
+        # Parse Model
+        parsed_data_model = data_model_parser.parse_model()
+
+        # Instantiate DataModelGraph
+        data_model_grapher = DataModelGraph(parsed_data_model, data_model_labels)
+
+        # Generate graph
+        self.graph_data_model = data_model_grapher.graph
+
+        self.dmge = DataModelGraphExplorer(self.graph_data_model)
 
         # check if the type of MModel file is "local"
         # currently, the application only supports reading from local JSON-LD files
@@ -102,7 +108,7 @@ class MetadataModel(object):
         Raises:
             ValueError: rootNode not found in metadata model.
         """
-        ordered_nodes = self.sg.get_descendants_by_edge_type(
+        ordered_nodes = self.dmge.get_descendants_by_edge_type(
             rootNode, relationshipType, connected=True, ordered=True
         )
 
@@ -140,6 +146,7 @@ class MetadataModel(object):
 
         mg = ManifestGenerator(
             path_to_json_ld=self.inputMModelLocation,
+            graph=self.graph_data_model,
             title=title,
             root=rootNode,
             additional_metadata=additionalMetadata,
@@ -169,11 +176,11 @@ class MetadataModel(object):
         """
 
         # get required components for the input/source component
-        req_components = self.sg.get_component_requirements(source_component)
+        req_components = self.dmge.get_component_requirements(source_component)
 
         # retreive components as graph
         if as_graph:
-            req_components_graph = self.sg.get_component_requirements_graph(
+            req_components_graph = self.dmge.get_component_requirements_graph(
                 source_component
             )
 
@@ -185,9 +192,17 @@ class MetadataModel(object):
         return req_components
 
     # TODO: abstract validation in its own module
+    @tracer.start_as_current_span("MetadataModel::validateModelManifest")
     def validateModelManifest(
-        self, manifestPath: str, rootNode: str, restrict_rules: bool = False, jsonSchema: str = None, project_scope: List = None,
-    ) -> List[str]:
+        self,
+        manifestPath: str,
+        rootNode: str,
+        restrict_rules: bool = False,
+        jsonSchema: Optional[str] = None,
+        project_scope: Optional[List] = None,
+        dataset_scope: Optional[str] = None,
+        access_token: Optional[str] = None,
+    ) -> tuple[list, list]:
         """Check if provided annotations manifest dataframe satisfies all model requirements.
 
         Args:
@@ -203,21 +218,29 @@ class MetadataModel(object):
             ValueError: rootNode not found in metadata model.
         """
         # get validation schema for a given node in the data model, if the user has not provided input validation schema
-        
+
         if not jsonSchema:
-            jsonSchema = self.sg.get_json_schema_requirements(
+            # Instantiate Data Model Json Schema
+            self.data_model_js = DataModelJSONSchema(
+                jsonld_path=self.inputMModelLocation, graph=self.graph_data_model
+            )
+
+            jsonSchema = self.data_model_js.get_json_validation_schema(
                 rootNode, rootNode + "_validation"
             )
 
         errors = []
         warnings = []
 
-        load_args={
-            "dtype":"string",
-            }
+        load_args = {
+            "dtype": "string",
+        }
         # get annotations from manifest (array of json annotations corresponding to manifest rows)
         manifest = load_df(
-            manifestPath, preserve_raw_input=False, **load_args,
+            manifestPath,
+            preserve_raw_input=False,
+            allow_na_values=True,
+            **load_args,
         )  # read manifest csv file as is from manifest path
 
         # handler for mismatched components/data types
@@ -251,10 +274,24 @@ class MetadataModel(object):
 
             return errors, warnings
 
-        errors, warnings, manifest = validate_all(self, errors, warnings, manifest, manifestPath, self.sg, jsonSchema, restrict_rules, project_scope)
+        errors, warnings, manifest = validate_all(
+            self,
+            errors=errors,
+            warnings=warnings,
+            manifest=manifest,
+            manifestPath=manifestPath,
+            dmge=self.dmge,
+            jsonSchema=jsonSchema,
+            restrict_rules=restrict_rules,
+            project_scope=project_scope,
+            dataset_scope=dataset_scope,
+            access_token=access_token,
+        )
         return errors, warnings
 
-    def populateModelManifest(self, title, manifestPath: str, rootNode: str, return_excel = False) -> str:
+    def populateModelManifest(
+        self, title, manifestPath: str, rootNode: str, return_excel=False
+    ) -> str:
         """Populate an existing annotations manifest based on a dataframe.
             TODO: Remove this method; always use getModelManifest instead
 
@@ -269,94 +306,132 @@ class MetadataModel(object):
             ValueError: rootNode not found in metadata model.
         """
         mg = ManifestGenerator(
-            path_to_json_ld=self.inputMModelLocation, title=title, root=rootNode
+            path_to_data_model=self.inputMModelLocation,
+            graph=self.graph_data_model,
+            title=title,
+            root=rootNode,
         )
 
         emptyManifestURL = mg.get_manifest()
 
-        return mg.populate_manifest_spreadsheet(manifestPath, emptyManifestURL, return_excel = return_excel, title=title)
+        return mg.populate_manifest_spreadsheet(
+            manifestPath, emptyManifestURL, return_excel=return_excel, title=title
+        )
 
-    def submit_metadata_manifest(
+    @tracer.start_as_current_span("MetadataModel::submit_metadata_manifest")
+    def submit_metadata_manifest(  # pylint: disable=too-many-arguments, too-many-locals
         self,
         manifest_path: str,
-        path_to_json_ld: str,
         dataset_id: str,
         manifest_record_type: str,
         restrict_rules: bool,
-        validate_component: str = None,
-        use_schema_label: bool = True,
+        access_token: Optional[str] = None,
+        validate_component: Optional[str] = None,
+        file_annotations_upload: bool = True,
         hide_blanks: bool = False,
-        input_token: str = None,
-        project_scope: List = None,
-    ) -> string:
-        """Wrap methods that are responsible for validation of manifests for a given component, and association of the
-        same manifest file with a specified dataset.
+        project_scope: Optional[list] = None,
+        dataset_scope: Optional[str] = None,
+        table_manipulation: str = "replace",
+        table_column_names: str = "class_label",
+        annotation_keys: str = "class_label",
+    ) -> str:
+        """
+        Wrap methods that are responsible for validation of manifests for a given component,
+          and association of the same manifest file with a specified dataset.
+
         Args:
-            manifest_path: Path to the manifest file, which contains the metadata.
-            dataset_id: Synapse ID of the dataset on Synapse containing the metadata manifest file.
-            validate_component: Component from the schema.org schema based on which the manifest template has been generated.
-        Returns:
-            Manifest ID: If both validation and association were successful.
-        Exceptions:
+            manifest_path (str): Path to the manifest file, which contains the metadata.
+            dataset_id (str): Synapse ID of the dataset on Synapse containing the
+              metadata manifest file.
+            manifest_record_type (str): How the manifest is stored in Synapse
+            restrict_rules (bool):
+              If True: bypass great expectations and restrict rule options to
+                those implemented in house
+            access_token (Optional[str], optional): Defaults to None.
+            validate_component (Optional[str], optional): Component from the schema.org
+              schema based on which the manifest template has been generated.
+            file_annotations_upload (bool, optional): Default to True. If false, do
+              not add annotations to files. Defaults to True.
+            hide_blanks (bool, optional): Defaults to False.
+            project_scope (Optional[list], optional): Defaults to None.
+            table_manipulation (str, optional): Defaults to "replace".
+            table_column_names (str, optional): Defaults to "class_label".
+            annotation_keys (str, optional): Defaults to "class_label".
+
+        Raises:
             ValueError: When validate_component is provided, but it cannot be found in the schema.
             ValidationError: If validation against data model was not successful.
-        """
 
-        #TODO: avoid explicitly exposing Synapse store functionality
+        Returns:
+            str: If both validation and association were successful.
+        """
+        # TODO: avoid explicitly exposing Synapse store functionality
         # just instantiate a Store class and let it decide at runtime/config
         # the store type
-        syn_store = SynapseStorage(input_token = input_token, project_scope = project_scope)
-        manifest_id=None
-        censored_manifest_id=None
-        restrict_maniest=False
-        censored_manifest_path=manifest_path.replace('.csv','_censored.csv')
+        syn_store = SynapseStorage(
+            access_token=access_token, project_scope=project_scope
+        )
+        manifest_id = None
+        restrict_maniest = False
+        censored_manifest_path = manifest_path.replace(".csv", "_censored.csv")
         # check if user wants to perform validation or not
         if validate_component is not None:
-
             try:
-                # check if the component ("class" in schema) passed as argument is valid (present in schema) or not
-                self.sg.se.is_class_in_schema(validate_component)
-            except:
-                # a KeyError exception is raised when validate_component fails in the try-block above
-                # here, we are suppressing the KeyError exception and replacing it with a more
-                # descriptive ValueError exception
+                # check if the component ("class" in schema) passed as argument is valid
+                # (present in schema) or not
+                self.dmge.is_class_in_schema(validate_component)
+            except Exception as exc:
+                # a KeyError exception is raised when validate_component fails in the
+                # try-block above here, we are suppressing the KeyError exception and
+                # replacing it with a more descriptive ValueError exception
                 raise ValueError(
                     f"The component '{validate_component}' could not be found "
-                    f"in the schema here '{path_to_json_ld}'"
-                )
+                    f"in the schema here '{self.path_to_json_ld}'"
+                ) from exc
 
             # automatic JSON schema generation and validation with that JSON schema
-            val_errors, val_warnings = self.validateModelManifest(
-                manifestPath=manifest_path, rootNode=validate_component, restrict_rules=restrict_rules, project_scope=project_scope,
+            val_errors, _ = self.validateModelManifest(
+                manifestPath=manifest_path,
+                rootNode=validate_component,
+                restrict_rules=restrict_rules,
+                project_scope=project_scope,
+                dataset_scope=dataset_scope,
+                access_token=access_token,
             )
 
             # if there are no errors in validation process
-            if val_errors == []:                
+            if val_errors == []:
                 # upload manifest file from `manifest_path` path to entity with Syn ID `dataset_id`
-                if exists(censored_manifest_path):
-                    censored_manifest_id = syn_store.associateMetadataWithFiles(
-                        schemaGenerator = self.sg,
-                        metadataManifestPath = censored_manifest_path,
-                        datasetId = dataset_id, 
-                        manifest_record_type = manifest_record_type,
-                        useSchemaLabel = use_schema_label,
-                        hideBlanks = hide_blanks,
+                if os.path.exists(censored_manifest_path):
+                    syn_store.associateMetadataWithFiles(
+                        dmge=self.dmge,
+                        metadataManifestPath=censored_manifest_path,
+                        datasetId=dataset_id,
+                        manifest_record_type=manifest_record_type,
+                        hideBlanks=hide_blanks,
+                        table_manipulation=table_manipulation,
+                        table_column_names=table_column_names,
+                        annotation_keys=annotation_keys,
+                        file_annotations_upload=file_annotations_upload,
                     )
                     restrict_maniest = True
-                
+
                 manifest_id = syn_store.associateMetadataWithFiles(
-                    schemaGenerator = self.sg,
-                    metadataManifestPath = manifest_path, 
-                    datasetId = dataset_id, 
-                    manifest_record_type = manifest_record_type,
-                    useSchemaLabel = use_schema_label, 
-                    hideBlanks = hide_blanks,
+                    dmge=self.dmge,
+                    metadataManifestPath=manifest_path,
+                    datasetId=dataset_id,
+                    manifest_record_type=manifest_record_type,
+                    hideBlanks=hide_blanks,
                     restrict_manifest=restrict_maniest,
+                    table_manipulation=table_manipulation,
+                    table_column_names=table_column_names,
+                    annotation_keys=annotation_keys,
+                    file_annotations_upload=file_annotations_upload,
                 )
 
-                logger.info(f"No validation errors occured during validation.")
+                logger.info("No validation errors occured during validation.")
                 return manifest_id
-                
+
             else:
                 raise ValidationError(
                     "Manifest could not be validated under provided data model. "
@@ -364,25 +439,31 @@ class MetadataModel(object):
                 )
 
         # no need to perform validation, just submit/associate the metadata manifest file
-        if exists(censored_manifest_path):
-            censored_manifest_id = syn_store.associateMetadataWithFiles(
-                schemaGenerator = self.sg,
+        if os.path.exists(censored_manifest_path):
+            syn_store.associateMetadataWithFiles(
+                dmge=self.dmge,
                 metadataManifestPath=censored_manifest_path,
                 datasetId=dataset_id,
                 manifest_record_type=manifest_record_type,
-                useSchemaLabel=use_schema_label,
                 hideBlanks=hide_blanks,
+                table_manipulation=table_manipulation,
+                table_column_names=table_column_names,
+                annotation_keys=annotation_keys,
+                file_annotations_upload=file_annotations_upload,
             )
             restrict_maniest = True
-        
+
         manifest_id = syn_store.associateMetadataWithFiles(
-            schemaGenerator = self.sg,
+            dmge=self.dmge,
             metadataManifestPath=manifest_path,
             datasetId=dataset_id,
             manifest_record_type=manifest_record_type,
-            useSchemaLabel=use_schema_label,
             hideBlanks=hide_blanks,
             restrict_manifest=restrict_maniest,
+            table_manipulation=table_manipulation,
+            table_column_names=table_column_names,
+            annotation_keys=annotation_keys,
+            file_annotations_upload=file_annotations_upload,
         )
 
         logger.debug(
